from django import forms
from django.utils.translation import ugettext_lazy as _

from discussion.models import Comment, Post, Discussion
from notification.models import NoticeSetting

from .utils import file_extension, DISCUSSION_UPLOAD_EXTENSIONS


def _clean_attachment(self):
    data = self.cleaned_data['attachment']
    if data is None:
        return data
    extension = file_extension(data.name)
    try:
        extension_ok = extension in DISCUSSION_UPLOAD_EXTENSIONS
    except TypeError:
        pass  # DISCUSSION_UPLOAD_EXTENSIONS has been set to None.
    else:
        if not extension_ok:
            msg = _('This is not an acceptable file type!')
            raise forms.ValidationError(msg)
    return data


class CommentForm(forms.ModelForm):
    class Meta:
        exclude = ('user', 'post')
        model = Comment
        widgets = {
            'body': forms.Textarea(attrs={'placeholder': _('Reply to this conversation')}),
        }
    clean_attachment = _clean_attachment


class PostForm(forms.ModelForm):
    class Meta:
        exclude = ('user', 'discussion')
        model = Post
        widgets = {
            'body': forms.Textarea(attrs={'placeholder': _('Start a conversation')}),
        }
    clean_attachment = _clean_attachment


class SearchForm(forms.Form):
    search = forms.CharField()
    discussion = forms.ModelChoiceField(
            required=False,
            queryset=Discussion.objects,
            empty_label=_('All discussions'))


class SubscribeForm(forms.Form):
    send = forms.ModelMultipleChoiceField(
            NoticeSetting.objects,
            required=False,
            label=_('Notify me with updates from this discussion by'),
            widget=forms.CheckboxSelectMultiple)

    def __init__(self, *args, **kwargs):
        qs = kwargs.pop('qs')
        super(SubscribeForm, self).__init__(*args, **kwargs)
        self.fields['send'].queryset = qs



import os

from django.contrib.auth.models import User
from django.db import models
from django.template.defaultfilters import date, time
from django.db.models.signals import post_save, pre_delete
from django.dispatch import receiver

from notification.models import NoticeType, create_notice_type, send
from orderable.models import Orderable


class Discussion(Orderable):
    user = models.ForeignKey(User)
    name = models.CharField(max_length=255)
    slug = models.SlugField()
    image = models.ImageField(upload_to='images/discussions', max_length=255, blank=True, null=True)
    description = models.TextField(default='', blank=True, null=True)

    def __unicode__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        return ('discussion', [self.slug])

    @property
    def notification_label(self):
        return "discussion_remind_%s" % self.slug

    @property
    def notification_display(self):
        return ("Notification for %s." % self.name)[:50]

    @property
    def notice_type(self):
        """
            Return the notice type for this discussion.
            If it does not exist then the notice type will be created.
        """
        self.create_notice_type()
        return NoticeType.objects.get(label=self.notification_label)

    def create_notice_type(self):
        """Create (or update) a notice type for discussion instance."""
        create_notice_type(
                label=self.notification_label,
                display=self.notification_display,
                description="A new post has been added .",
                slug=self._meta.app_label,
                default=0)


class Post(models.Model):
    discussion = models.ForeignKey(Discussion)
    user = models.ForeignKey(User)
    body = models.TextField()
    attachment = models.FileField(upload_to='uploads/posts', blank=True, null=True)
    time = models.DateTimeField(auto_now_add=True)

    class Meta:
        ordering = ('-time',)

    def __unicode__(self):
        return 'Post by {user} at {time} on {date}'.format(
            user=self.user,
            time=time(self.time),
            date=date(self.time),
        )

    @models.permalink
    def get_absolute_url(self):
        return ('discussion_post', [self.discussion.slug, str(self.id)])

    @property
    def attachment_filename(self):
        return self.attachment and os.path.basename(self.attachment.name)

    @property
    def prefix(self):
        return 'post-%d' % (self.pk or 0,)


class Comment(models.Model):
    post = models.ForeignKey(Post)
    user = models.ForeignKey(User)
    body = models.TextField()
    attachment = models.FileField(upload_to='uploads/comments',
                                    blank=True, null=True)
    time = models.DateTimeField(auto_now_add=True)

    @property
    def attachment_filename(self):
        return self.attachment and os.path.basename(self.attachment.name)

    class Meta:
        ordering = ('time',)

    def __unicode__(self):
        return 'Comment by {user} at {time} on {date}'.format(
            user=self.user,
            time=time(self.time),
            date=date(self.time),
        )


@receiver(post_save, sender=Discussion)
def create_discussion_notice_type(sender, instance, **kwargs):
    # If display or diary title has changed notification details will be updated or created for new diary.
    instance.create_notice_type()


@receiver(pre_delete, sender=Discussion)
def delete_discussion_notice_type(sender, instance, **kwargs):
    instance.notice_type.delete()


def notify_discussion_subscribers(discussion, instance):
    """
        Notifies all users who have their notification settings set to True for given discussion
        instance parameter here is either Post or Comment
    """
    notification_label = discussion.notification_label
    users = User.objects.filter(
            noticesetting__send=True,
            noticesetting__notice_type__label=notification_label).exclude(id=instance.user.id).distinct()
    send(users, notification_label, {'discussion': discussion}, now=True)


def post_notifications(sender, instance, created, **kwargs):
    if created:
        notify_discussion_subscribers(instance.discussion, instance)
models.signals.post_save.connect(post_notifications, sender=Post)


def comment_notifications(sender, instance, created, **kwargs):
    if created:
        notify_discussion_subscribers(instance.post.discussion, instance)
models.signals.post_save.connect(comment_notifications, sender=Comment)



from haystack import indexes
from .models import Discussion, Post


class DiscussionIndex(indexes.SearchIndex, indexes.Indexable):
    title = indexes.CharField(model_attr='name')
    text = indexes.CharField(document=True, use_template=True)

    def get_model(self):
        return Discussion

    def index_queryset(self):
        return self.get_model().objects.all()


class PostIndex(indexes.SearchIndex, indexes.Indexable):
    title = indexes.CharField(use_template=True)
    text = indexes.CharField(document=True, use_template=True)

    def get_model(self):
        return Post

    def index_queryset(self):
        return self.get_model().objects.all()



from django.utils.decorators import method_decorator
from django.conf import settings


def class_view_decorator(function_decorator):
    """Convert a function based decorator into a class based decorator usable
    on class based Views.

    Follows the general idea from https://docs.djangoproject.com/en/dev/topics/class-based-views/#decorating-the-class.

    Can't subclass the `View` as it breaks inheritance (super in particular),
    so we monkey-patch instead.
    """

    def simple_decorator(View):
        View.dispatch = method_decorator(function_decorator)(View.dispatch)
        return View

    return simple_decorator


# If this is set to `[]`, no uploads will be accepted.
# If it is set to `None`, the whitelist will not be used.
DISCUSSION_UPLOAD_EXTENSIONS = getattr(settings, 'DISCUSSION_UPLOAD_EXTENSIONS', [
    'odt', 'odf', 'odp',  # Open/LibreOffice
    'doc', 'docx', 'xls', 'xlsx', 'ppt', 'pptx',  # MS Office
    'pdf', 'gif', 'jpg', 'jpeg', 'png',  # Other sane defaults...
])


def file_extension(path):
    """Get the file extension of a path/filename."""
    return path.split('/')[-1].split('.')[-1]



from os.path import dirname, join
from setuptools import find_packages, setup

from discussion import get_version


def fread(fname):
    with open(join(dirname(__file__), fname), 'r') as f:
        return f.read()

setup(
    name='django-discussion',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'django-orderable>=1.0.1',
        'incuna-notification>=0.3.2',
        ],
    version=get_version(),
    description='',
    author='Incuna Ltd',
    author_email='dev@incuna.com',
    url='http://incuna.com/',
    long_description=fread('README.rst'),
)



from django.contrib import admin
from discussion.models import Comment, Discussion, Post
from orderable.admin import OrderableAdmin


class CommentInline(admin.TabularInline):
    extra = 1
    model = Comment
    raw_id_fields = ('user',)


class PostAdmin(admin.ModelAdmin):
    inlines = (CommentInline,)
    list_filter = ('discussion',)
    list_display = ('discussion', 'time')


class DiscussionAdmin(OrderableAdmin):
    prepopulated_fields = {
        'slug': ('name',)
    }
    list_display = ('name', 'description')


admin.site.register(Discussion, DiscussionAdmin)
admin.site.register(Post, PostAdmin)
admin.site.register(Comment)
